@@ -3,17 +3,90 @@ module FastAlmostBandedMatrices
 import PrecompileTools: @setup_workload, @compile_workload
 
 using ArrayInterface, ArrayLayouts, BandedMatrices, ConcreteStructs, LazyArrays,
-      LinearAlgebra, MatrixFactorizations, Reexport
+    LinearAlgebra, MatrixFactorizations, Reexport
 
 import ArrayLayouts: MemoryLayout, sublayout, sub_materialize, MatLdivVec, materialize!,
-                     triangularlayout, triangulardata, zero!, _copyto!, colsupport,
-                     rowsupport, _qr, _qr!, _factorize, muladd!
+    triangularlayout, triangulardata, zero!, _copyto!, colsupport,
+    rowsupport, _qr, _qr!, _factorize, muladd!
 import BandedMatrices: _banded_qr!, bandeddata, banded_qr_lmul!
 import LinearAlgebra: ldiv!
 import MatrixFactorizations: QR, QRPackedQ, getQ, getR, QRPackedQLayout, AdjQRPackedQLayout
 
 @reexport using BandedMatrices
 
+# ------------------
+# DisjointRange - for zero-allocation colsupport
+# ------------------
+
+"""
+    DisjointRange{T}
+
+A lazy representation of the union of two ranges, supporting iteration and indexing
+without heap allocation.
+"""
+struct DisjointRange{T <: Integer, R1 <: AbstractUnitRange{T}, R2 <: AbstractUnitRange{T}} <:
+    AbstractVector{T}
+    r1::R1
+    r2::R2
+end
+
+Base.size(d::DisjointRange) = (length(d.r1) + length(d.r2),)
+Base.length(d::DisjointRange) = length(d.r1) + length(d.r2)
+
+@inline function Base.getindex(d::DisjointRange, i::Integer)
+    @boundscheck checkbounds(d, i)
+    n1 = length(d.r1)
+    if i <= n1
+        return @inbounds d.r1[i]
+    else
+        return @inbounds d.r2[i - n1]
+    end
+end
+
+Base.IndexStyle(::Type{<:DisjointRange}) = IndexLinear()
+
+@inline function Base.iterate(d::DisjointRange)
+    if !isempty(d.r1)
+        val, state = iterate(d.r1)
+        return val, (1, state)
+    elseif !isempty(d.r2)
+        val, state = iterate(d.r2)
+        return val, (2, state)
+    else
+        return nothing
+    end
+end
+
+@inline function Base.iterate(d::DisjointRange, state)
+    which, inner_state = state
+    return if which == 1
+        next = iterate(d.r1, inner_state)
+        if next !== nothing
+            return next[1], (1, next[2])
+        else
+            # Switch to r2
+            if !isempty(d.r2)
+                val, new_state = iterate(d.r2)
+                return val, (2, new_state)
+            else
+                return nothing
+            end
+        end
+    else
+        next = iterate(d.r2, inner_state)
+        if next !== nothing
+            return next[1], (2, next[2])
+        else
+            return nothing
+        end
+    end
+end
+
+Base.first(d::DisjointRange) = isempty(d.r1) ? first(d.r2) : first(d.r1)
+Base.last(d::DisjointRange) = isempty(d.r2) ? last(d.r1) : last(d.r2)
+Base.minimum(d::DisjointRange) = min(minimum(d.r1), minimum(d.r2))
+Base.maximum(d::DisjointRange) = max(maximum(d.r1), maximum(d.r2))
+
 # ------------------
 # AlmostBandedMatrix
 # ------------------
@@ -51,28 +124,35 @@ part.
     fill
 end
 
-function AlmostBandedMatrix(::UndefInitializer, ::Type{T}, mn::NTuple{2, Integer},
-        lu::NTuple{2, Integer}, rank::Integer) where {T}
+function AlmostBandedMatrix(
+        ::UndefInitializer, ::Type{T}, mn::NTuple{2, Integer},
+        lu::NTuple{2, Integer}, rank::Integer
+    ) where {T}
     @assert lu[2] ≥ rank - 1
-    @assert rank≥1 "Rank 0 fill array makes it a BandedMatrix."
+    @assert rank ≥ 1 "Rank 0 fill array makes it a BandedMatrix."
     bands = BandedMatrix{T}(undef, mn, lu)
     fill = Matrix{T}(undef, rank, mn[2])
     return AlmostBandedMatrix{T}(bands, fill)
 end
 
-function AlmostBandedMatrix{T}(::UndefInitializer, mn::NTuple{2, Integer},
-        lu::NTuple{2, Integer}, rank::Integer) where {T}
+function AlmostBandedMatrix{T}(
+        ::UndefInitializer, mn::NTuple{2, Integer},
+        lu::NTuple{2, Integer}, rank::Integer
+    ) where {T}
     return AlmostBandedMatrix(undef, T, mn, lu, rank)
 end
 
 function AlmostBandedMatrix(
-        ::UndefInitializer, mn::NTuple{2, Integer}, lu::NTuple{2, Integer}, rank::Integer)
+        ::UndefInitializer, mn::NTuple{2, Integer}, lu::NTuple{
+            2, Integer,
+        }, rank::Integer
+    )
     return AlmostBandedMatrix(undef, Float64, mn, lu, rank)
 end
 
 function AlmostBandedMatrix(bands::BandedMatrix, fill::AbstractMatrix)
     @assert size(fill, 2) == size(bands, 2)
-    @assert size(fill, 1)≥1 "Rank 0 fill array makes it a BandedMatrix."
+    @assert size(fill, 1) ≥ 1 "Rank 0 fill array makes it a BandedMatrix."
     T = promote_type(eltype(fill), eltype(bands))
     @assert bandwidths(bands)[1] ≥ size(fill, 1) - 1
     finish_part_setindex!(bands, fill)
@@ -81,6 +161,17 @@ end
 
 MemoryLayout(::Type{<:AlmostBandedMatrix}) = AlmostBandedLayout()
 
+"""
+    finish_part_setindex!(A::AlmostBandedMatrix)
+    finish_part_setindex!(bands, fill)
+
+Synchronize the overlapping region between the banded part and the fill part of an
+`AlmostBandedMatrix`. This function copies values from the `fill` part into the
+corresponding positions in the `bands` part where they overlap.
+
+This is automatically called during construction but may need to be called manually if
+the fill part is modified directly after construction.
+"""
 @inline function finish_part_setindex!(A)
     return finish_part_setindex!(bandpart(A), fillpart(A))
 end
@@ -88,6 +179,7 @@ end
     # copy `fill` into `bands` in the correct locations
     l, u = bandwidths(bands)
     for i in 1:size(fill, 1), j in max(1, i - l):min(size(bands, 2), i + u)
+
         @inbounds bands[i, j] = fill[i, j]
     end
     return nothing
@@ -96,21 +188,44 @@ end
 """
     bandpart(A::AlmostBandedMatrix)
 
-Banded Part of the AlmostBandedMatrix.
+Return the banded part of the `AlmostBandedMatrix`. This is a `BandedMatrix` that stores
+the banded structure including the overlapping region with the fill part.
+
+# Example
+```julia
+A = AlmostBandedMatrix(brand(Float64, 10, 10, 3, 2), rand(Float64, 2, 10))
+B = bandpart(A)  # Returns the BandedMatrix component
+```
 """
 @inline bandpart(A::AlmostBandedMatrix) = A.bands
 
 """
     fillpart(A::AlmostBandedMatrix)
 
-Fill Part of the AlmostBandedMatrix.
+Return the fill part of the `AlmostBandedMatrix`. This is typically a low-rank matrix that
+represents additional structure beyond the banded part. The fill part overlaps with the
+first `almostbandedrank(A)` rows of the banded part.
+
+# Example
+```julia
+A = AlmostBandedMatrix(brand(Float64, 10, 10, 3, 2), rand(Float64, 2, 10))
+F = fillpart(A)  # Returns the fill matrix (2×10)
+```
 """
 @inline fillpart(A::AlmostBandedMatrix) = A.fill
 
 """
     exclusive_bandpart(A::AlmostBandedMatrix)
 
-Banded Part of the AlmostBandedMatrix without the overlapping part.
+Return the banded part of the `AlmostBandedMatrix` excluding the rows that overlap with
+the fill part. This is a view of the banded matrix starting from row
+`almostbandedrank(A) + 1`.
+
+# Example
+```julia
+A = AlmostBandedMatrix(brand(Float64, 10, 10, 3, 2), rand(Float64, 2, 10))
+E = exclusive_bandpart(A)  # Returns a view of rows 3:10 of the banded part
+```
 """
 @inline function exclusive_bandpart(A)
     B, F = bandpart(A), fillpart(A)
@@ -126,7 +241,7 @@ end
 @inline Base.eltype(::AlmostBandedMatrix{T}) where {T} = T
 @inline Base.IndexStyle(::Type{<:AlmostBandedMatrix}) = IndexCartesian()
 
-# If dims is provided we will construct a Matrix but other invokations should return a
+# If dims is provided we will construct a Matrix but other invocations should return a
 # AlmostBandedMatrix
 function Base.similar(A::AlmostBandedMatrix, ::Type{T}) where {T}
     bands = similar(A.bands, T)
@@ -137,7 +252,7 @@ end
 function Base.fill!(A::AlmostBandedMatrix, v)
     fill!(bandpart(A), v)
     fill!(fillpart(A), v)
-    return nothing
+    return A
 end
 
 @inline function colsupport(::AbstractAlmostBandedLayout, A, j)
@@ -150,7 +265,8 @@ end
         if isempty(sup)
             return Base.OneTo(r)
         else
-            return vcat(Base.OneTo(min(r, minimum(sup) - 1)), sup)
+            # Use DisjointRange to avoid heap allocation from vcat
+            return DisjointRange(Base.OneTo(min(r, minimum(sup) - 1)), sup)
         end
     end
 end
@@ -199,8 +315,13 @@ function LinearAlgebra.triu!(A::AlmostBandedMatrix)
 end
 
 # TODO: Support views properly
-function sublayout(::AlmostBandedLayout,
-        ::Type{<:Tuple{AbstractUnitRange{Int}, AbstractUnitRange{Int}}})
+function sublayout(
+        ::AlmostBandedLayout, ::Type{
+            <:Tuple{
+                AbstractUnitRange{Int}, AbstractUnitRange{Int},
+            },
+        }
+    )
     return AlmostBandedLayout()
 end
 
@@ -235,16 +356,18 @@ end
 # Pretty Printing
 # ---------------
 function _almost_banded_summary(io, B::AlmostBandedMatrix{T}, inds) where {T}
-    print(io, Base.dims2string(length.(inds)),
+    return print(
+        io, Base.dims2string(length.(inds)),
         " AlmostBandedMatrix{$T} with bandwidths $(almostbandwidths(B)) and fill \
-          rank $(almostbandedrank(B))")
+          rank $(almostbandedrank(B))"
+    )
 end
 function Base.array_summary(io::IO, B::AlmostBandedMatrix, inds::Tuple{Vararg{Base.OneTo}})
     _almost_banded_summary(io, B, inds)
     print(io, " with data ")
     summary(io, B.bands)
     print(io, " and fill ")
-    summary(io, B.fill)
+    return summary(io, B.fill)
 end
 function Base.array_summary(io::IO, B::AlmostBandedMatrix, inds)
     _almost_banded_summary(io, B, inds)
@@ -252,7 +375,7 @@ function Base.array_summary(io::IO, B::AlmostBandedMatrix, inds)
     summary(io, B.bands)
     print(io, " and fill ")
     summary(io, B.fill)
-    print(io, " with indices ", Base.inds2string(inds))
+    return print(io, " with indices ", Base.inds2string(inds))
 end
 
 # --------------
@@ -261,7 +384,7 @@ end
 
 function ArrayInterface.fast_scalar_indexing(A::AlmostBandedMatrix)
     return ArrayInterface.fast_scalar_indexing(typeof(A.bands)) &&
-           ArrayInterface.fast_scalar_indexing(typeof(A.fill))
+        ArrayInterface.fast_scalar_indexing(typeof(A.fill))
 end
 
 function ArrayInterface.qr_instance(A::AlmostBandedMatrix{T}, pivot = NoPivot()) where {T}
@@ -283,8 +406,8 @@ function _almostbanded_qr(_, A)
     # Expand the bandsize for the QR factorization
     ## Bypass the safety checks in `AlmostBandedMatrix`
     return almostbanded_qr!(
-        AlmostBandedMatrix{eltype(A)}(BandedMatrix(copy(B), (l, l + u)), copy(L)),
-        Val(true))
+        AlmostBandedMatrix{eltype(A)}(BandedMatrix(copy(B), (l, l + u)), copy(L)), Val(true)
+    )
 end
 
 # Band size not yet expanded!
@@ -337,13 +460,15 @@ end
         U′ = U[kr, :]
         for j in 1:length(jr2)
             muladd!(
-                -one(T), U′[(j + 1):end, :], L_right[:, j], one(T), B_right[(j + 1):end, j])
+                -one(T), U′[(j + 1):end, :], L_right[:, j], one(T), B_right[(j + 1):end, j]
+            )
         end
         banded_qr_lmul!(Q', B_right)
         banded_qr_lmul!(Q', U′)
         for j in 1:length(jr2)
             muladd!(
-                one(T), U′[(j + 1):end, :], L_right[:, j], one(T), B_right[(j + 1):end, j])
+                one(T), U′[(j + 1):end, :], L_right[:, j], one(T), B_right[(j + 1):end, j]
+            )
         end
         k = last(jr1) + 1
     end
@@ -379,11 +504,11 @@ _almostbanded_widerect_ldiv!(::QR{T}, B) where {T} = error("Not implemented")
 
 const UpperLayoutMatrix{T} = UpperTriangular{T, <:LayoutMatrix{T}}
 
-for Typ in (
-    :StridedVector, :StridedMatrix, :AbstractVecOrMat, :UpperLayoutMatrix, :LayoutMatrix)
+for Typ in
+    (:StridedVector, :StridedMatrix, :AbstractVecOrMat, :UpperLayoutMatrix, :LayoutMatrix)
     @eval function ldiv!(A::QR{T, <:AlmostBandedMatrix}, B::$Typ{T}) where {T}
         m, n = size(A)
-        if m == n
+        return if m == n
             _almostbanded_square_ldiv!(A, B)
         elseif n > m
             _almostbanded_widerect_ldiv!(A, B)
@@ -427,7 +552,8 @@ end
 @inline __original_almostbandedrank(A) = size(first(__lowrankfillpart(A)), 2)
 
 @views function _almostbanded_upper_ldiv!(
-        ::Type{Tri}, R::AbstractMatrix, b::AbstractVector{T}, buffer) where {T, Tri}
+        ::Type{Tri}, R::AbstractMatrix, b::AbstractVector{T}, buffer
+    ) where {T, Tri}
     B = bandpart(R)
     U, V = __lowrankfillpart(R)
     fill!(buffer, zero(T))
@@ -489,7 +615,8 @@ end
 # ---------------
 
 @views function muladd!(
-        α, A::AlmostBandedMatrix, B::AbstractVecOrMat, β, C::AbstractVecOrMat)
+        α, A::AlmostBandedMatrix, B::AbstractVecOrMat, β, C::AbstractVecOrMat
+    )
     L = fillpart(A)
     muladd!(α, L, B, β, selectdim(C, 1, 1:size(L, 1)))
     muladd!(α, exclusive_bandpart(A), B, β, selectdim(C, 1, (size(L, 1) + 1):size(C, 1)))
@@ -520,6 +647,6 @@ end
 end
 
 export AlmostBandedMatrix, bandpart, fillpart, exclusive_bandpart, finish_part_setindex!,
-       almostbandwidths, almostbandedrank
+    almostbandwidths, almostbandedrank
 
 end
